<?php
/**
 * Plugin Name: Shipping Zones by Drawing for WooCommerce
 * Plugin URI: https://shippingzonesplugin.com
 * Description: Limit WooCommerce shipping methods by a drawn shipping zone or by a transportation radius.
 * Version: 2.8.4.1
 * Author: Arosoft.se
 * Author URI: https://arosoft.se
 * Developer: Arosoft.se
 * Developer URI: https://arosoft.se
 * Text Domain: szbd
 * Domain Path: /languages
 * WC requires at least: 5.0
 * WC tested up to: 7.1
 * Copyright: Arosoft.se 2022
 * License: GPL v2 or later
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
if (!defined('ABSPATH'))
  {
  exit;
  }

define('SZBD_VERSION', '2.8.4.1');
define('SZBD_PLUGINDIRURL', plugin_dir_url(__FILE__));
define('SZBD_PLUGINDIRPATH', plugin_dir_path(__FILE__));

// Register hook for activation
register_activation_hook(__FILE__, array(
  'SZBD',
  'activate'
));

// Register hook for deactivation
register_deactivation_hook(__FILE__, array(
    'SZBD',
    'deactivate'
));

if (!class_exists('SZBD'))
  {
  /**
   * Main Class SZBD
   *
   * @since 1.0.0
   */

  class SZBD
    {
    const TEXT_DOMAIN = 'szbd';
    const POST_TITLE = 'szbdzones';
    protected static $_instance = null;
    public $notices;
    public $products;
    static $store_address;

    // to be run on plugin activation
    static public function activate()
      {
       if( get_option('szbd_precise_address','no') == 'always'){
        update_option('szbd_precise_address','at_fail');
       }
      require_once (SZBD_PLUGINDIRPATH . 'classes/class-szbd-the-post.php');
          
            
            $zonesObj = new Sbdzones_Post();
            $zonesObj->register_post_szbdzones();
            
            self::set_admin_caps();

            flush_rewrite_rules();
      }

    // to be run on plugin deactivation
    public static function deactivate()
      {

      unregister_post_type('szbdzones');
      flush_rewrite_rules();
      }
      
      static function set_admin_caps(){
        
        $admin_capabilities = array(
                'delete_szbdzones',
                'delete_others_szbdzones',
                'delete_private_szbdzones',
                'delete_published_szbdzones',
                'edit_szbdzones',
                'edit_others_szbdzones',
                'edit_private_szbdzones',
                'edit_published_szbdzones',
                'publish_szbdzones',
                'read_private_szbdzones',
               
            );
            $admin = get_role('administrator');
            foreach ($admin_capabilities as $capability) {
                $admin->add_cap($capability);
            }
      }

    public static function instance()
      {
      NULL === self::$_instance and self::$_instance = new self;
      return self::$_instance;
      }

    // The Constructor
    public function __construct()
      {
      add_action('wp', array(
                $this,
                'check_conditionals'
            ));
  add_action('woocommerce_checkout_update_order_review', 'szbd_clear_wc_shipping_rates_cache');
  add_action('woocommerce_checkout_update_order_review', 'szbd_clear_session');
  add_action('szbd_clear_session', 'szbd_clear_session');
  
  add_action('szbd_clear_shipping_rates_cache', 'szbd_clear_wc_shipping_rates_cache');
   
   // Update shipping rates when updating shipping address from blocks checkout
            add_action( 'woocommerce_store_api_cart_update_customer_from_request', function(){
                
            if(get_option('szbd_server_mode','no') == 'yes'){
                szbd_clear_session();
                szbd_clear_wc_shipping_rates_cache();
                add_filter( 'woocommerce_package_rates', array($this, 'szbd_filter_shipping_methods_for_checkout_server_mode'), 999 );
              }
            });
  
      add_filter('plugin_action_links_' . plugin_basename(__FILE__) , array(
        $this,
        'add_action_links'
      ));
      add_action('init', array(
        $this,
        'load_text_domain'
      ));
      add_action('admin_init', array(
        $this,
        'check_environment'
      ));
      add_action('plugins_loaded', array(
        $this,
        'init'
      ) , 10);
      add_action('admin_notices', array(
        $this,
        'admin_notices'
      ) , 15);
      add_action('wp_enqueue_scripts', array(
        $this,
        'maybe_enqueue_scripts'
      ) , 998);
      add_action( 'enqueue_block_assets', array( $this, 'enqueue_checkout_block_assets' ));
      add_action('wp', array(
        $this,
        'init_shortcode'
      ));
      add_filter('manage_edit-szbdzones_columns', array(
        $this,
        'posts_columns_id'
      ) , 2);

      add_action('manage_posts_custom_column', array(
        $this,
        'posts_custom_id_columns'
      ) , 5, 2);
      add_filter('woocommerce_cart_ready_to_calc_shipping', array($this, 'szbd_disable_shipping_calc_on_cart'), 999);


       $placement = get_option('szbd_precise_address', 'no') != 'no' ? get_option('szbd_map_placement', 'before_payment') : 'none';
            switch ($placement) {
                case 'none':
                break;
                case 'before_details':
                    add_action('woocommerce_checkout_before_customer_details', array(
                        $this,
                        'insert_to_checkout'
                    ));
                break;
                case 'before_payment':
                    add_action('woocommerce_review_order_before_payment', array(
                        $this,
                        'insert_to_checkout'
                    ) , 99);
                break;
                case 'after_order_notes':
                    add_action('woocommerce_after_order_notes', array(
                        $this,
                        'insert_to_checkout'
                    ) , 99);
                break;
                case 'before_order_review':
                    add_action('woocommerce_checkout_before_order_review', array(
                        $this,
                        'insert_to_checkout'
                    ) , 99);
                break;
             case 'after_billing_form':
                    add_action('woocommerce_after_checkout_billing_form', array(
                        $this,
                        'insert_to_checkout'
                    ) , 99);
                break;
               
               
              
                break;
             
            }
      add_action('woocommerce_checkout_update_order_meta', array(
        $this,
        'szbd_checkout_field_update_order_meta'
      ) , 10, 1);

      add_action('woocommerce_admin_order_data_after_billing_address', array(
        $this,
        'szbd_show_checkout_field_admin_order_meta'
      ) , 10, 1);

      add_action('woocommerce_email_order_meta', array(
        $this,
        'szbd_add_picked_location_to_emails'
      ) , 20, 4);

      add_action('woocommerce_thankyou', array(
        $this,
        'szbd_add_picked_location_to_thankyou_page'
      ) , 99, 1);

        add_action('woocommerce_checkout_process', array($this,'validate_checkout_field_process'));

      add_action( 'woocommerce_admin_order_preview_start', array(
        $this,
        'szbd_preview_meta') );


      }
      function check_conditionals(){
        if(get_option('szbd_server_mode','no') == 'yes'){
                
                 add_filter( 'woocommerce_package_rates', array($this, 'szbd_filter_shipping_methods_for_checkout_server_mode'), 999 );
                 
                
                  
                 if( is_cart() && get_option('szbd_enable_at_cart','no') == 'yes'){
                  
                   add_action( 'woocommerce_before_calculate_totals', 'szbd_clear_session');
                   
                    add_filter( 'woocommerce_cart_shipping_packages', array($this,'wc_shipping_rate_cache_invalidation'), 100 );
                    


                 }
                  
                
            
            }
        
        }
         function wc_shipping_rate_cache_invalidation( $packages ) {
   
	foreach ( $packages as &$package ) {
		$package['rate_cache'] = wp_rand(0,1000);
	}

	return $packages;
}
       public function maybe_enqueue_scripts()
      {
          if(get_option('szbd_server_mode','no') == 'no'){
             self::enqueue_scripts_aro();
            }else{
                self::enque_scripts_server_mode();
            }
            if (wc_post_content_has_shortcode('szbd') || get_option('szbd_force_shortcode', 'no') == 'yes') {
                self::enqueue_shortcode_scripts();
            }
        }


      
     static function enqueue_scripts_aro() {

      $do_cart = get_option('szbd_enable_at_cart','no') == 'yes' && is_cart();

     if (!(is_checkout() || $do_cart) || WC()
      ->cart
      ->needs_shipping() === false ||  is_wc_endpoint_url( 'order-pay' )) {
      return;
    }

    if (class_exists('Food_Online_Del') && get_option('fdoe_enable_delivery_switcher', 'no') !== 'no' && get_option('fdoe_enable_delivery_switcher', 'no') !== 'only_pickup' && get_option('fdoe_disable_checkout_validation','no') !== 'yes' && (get_option('fdoe_skip_address_validation', 'no') !== 'yes' &&  'skip' !== WC()->session->get('fdoe_bypass_validation',false))) {
      return;
    }
      if( class_exists('Food_Online_Del') &&  get_option('fdoe_enable_delivery_switcher', 'no') !== 'no' &&  get_option('fdoe_enable_delivery_switcher', 'no') !== 'only_delivery' && ('local_pickup' == WC()->session->get('fdoe_shipping') || 'eathere' == WC()->session->get('fdoe_shipping')) ){
      return;
    }
     if($do_cart){
     add_action( 'woocommerce_calculated_shipping', 'szbd_clear_session' );
    }

     $country_pos = null;
if(get_option('szbd_precise_address','no') != 'no' ){

    $request = wp_remote_get( SZBD_PLUGINDIRURL . 'assets/json/countries.json' );


    if( !is_wp_error( $request ) ) {
        $body = wp_remote_retrieve_body( $request );
    $country_pos = json_decode( $body );
    }
}

 $to_localize = array(
        'countries' =>   $country_pos,
        'checkout_string_1' => __('There are no shipping options available. Please ensure that your address has been entered correctly, or contact us if you need any help.', 'woocommerce') ,
        'checkout_string_2' => __('Minimum order value is', 'szbd') ,
        'checkout_string_3' => __('You are too far away. We only make deliveries within', 'szbd') ,
        'checkout_string_4' => __('Some items in your cart don’t ship to your location', 'szbd') ,
        'cart_string_1' => __('More shipping alternatives may exist when a full shipping address is entered.', 'szbd') ,
        'no_marker_error' => __('You have to precise a location at the map', 'szbd') ,
        'store_address' => get_option('szbd_store_address_mode', 'geo_woo_store') == 'pick_store_address' ? json_decode(get_option('SZbD_settings_test', '') , true) : SZBD::get_store_address() ,

        'debug' => get_option('szbd_debug', 'no') == 'yes' ? 1 : 0,
        'deactivate_postcode' => get_option('szbd_deactivate_postcode', 'no') == 'yes' ? 1 : 0,
        'select_top_method' => get_option('szbd_select_top_method', 'no') == 'yes' ? 1 : 0,
        'store_address_picked' => get_option('szbd_store_address_mode', 'geo_woo_store') == 'pick_store_address' ? 1 : 0,
        'precise_address' => get_option('szbd_precise_address','no') == 'always' ? 'at_fail' : get_option('szbd_precise_address','no'),
        'nonce' => wp_create_nonce( 'szbd-script-nonce' ),
        'is_cart' => $do_cart ? 1 : 0 ,
        'is_checkout' => is_checkout() ? 1 : 0 ,
        'is_custom_types' => get_option('szbd_types_custom','no') == 'yes' ? 1 : 0 ,
        'result_types' => get_option('szbd_result_types', array(
                        "establishment",
                        "subpremise",
                        "premise",
                        "street_address",
                        "plus_code"
                    )) ,
        'no_map_types' => get_option('szbd_no_map_types', array(
                        "establishment",
                        "subpremise",
                        "premise",
                        "street_address",
                        "plus_code",
                        "route",
                        "intersection"
                    )) ,

      );

      if (( is_checkout() || $do_cart ) && get_option('szbd_deactivate_google', 'no') == 'no') {

        $google_api_key = get_option('szbd_google_api_key', '');

        wp_enqueue_script('szbd-google-autocomplete-2', 'https://maps.googleapis.com/maps/api/js?v=3&libraries=geometry,places&key=' . $google_api_key);


         if (WP_DEBUG === true) {
      wp_enqueue_script('shipping-del-aro', SZBD_PLUGINDIRURL . 'assets/szbd.js', array(
        'jquery',
        'wc-checkout',
        'szbd-google-autocomplete-2',
        'underscore'
      ) , SZBD_VERSION, true);
       }else{
         wp_enqueue_script('shipping-del-aro', SZBD_PLUGINDIRURL . 'assets/szbd.min.js', array(
        'jquery',
        'wc-checkout',
        'szbd-google-autocomplete-2',
        'underscore'
      ) , SZBD_VERSION, true);
       }
       wp_localize_script('shipping-del-aro', 'szbd', $to_localize);
        wp_enqueue_style('shipping-del-aro-style', SZBD_PLUGINDIRURL . 'assets/szbd.css', SZBD_VERSION);

      }
      else if ((is_checkout() || $do_cart ) && get_option('szbd_deactivate_google', 'no') == 'yes') {


         if (WP_DEBUG === true) {
      wp_enqueue_script('shipping-del-aro', SZBD_PLUGINDIRURL . 'assets/szbd.js', array(
        'jquery',
        'wc-checkout',

        'underscore'
      ) , SZBD_VERSION, true);
       }else{
         wp_enqueue_script('shipping-del-aro', SZBD_PLUGINDIRURL . 'assets/szbd.min.js', array(
        'jquery',
        'wc-checkout',

        'underscore'
      ) , SZBD_VERSION, true);
       }
        wp_localize_script('shipping-del-aro', 'szbd', $to_localize);
        wp_enqueue_style('shipping-del-aro-style', SZBD_PLUGINDIRURL . '/assets/szbd.css', SZBD_VERSION);

      }
    }
     function enqueue_checkout_block_assets(){
    
    if(class_exists('WC_Blocks_Utils') && WC_Blocks_Utils::has_block_in_page( get_the_ID(), 'woocommerce/checkout' )){
     if (WP_DEBUG === true) {
      wp_enqueue_script('szbd-checkout-block', SZBD_PLUGINDIRURL . 'assets/szbd-checkout-block.js', array(
        'jquery',
       
      ) , SZBD_VERSION, true);
       }else{
         wp_enqueue_script('szbd-checkout-block', SZBD_PLUGINDIRURL . 'assets/szbd-checkout-block.min.js', array(
        'jquery',
       
      ) , SZBD_VERSION, true);
       }
        $to_localize = array(
        'shipping_forced_to_billing' =>   get_option('woocommerce_ship_to_destination','') == 'billing_only' ? 1 : 0,
        );
        wp_localize_script('szbd-checkout-block', 'szbdblock', $to_localize);
    
    }
    
  }
      static function enque_scripts_server_mode(){
             

     if (!is_checkout()  || WC()
      ->cart
      ->needs_shipping() === false ||  is_wc_endpoint_url( 'order-pay' )) {
      return;
    }
    if(get_option('szbd_precise_address','no') == 'no' ){
        
        return;
    }

    if (class_exists('Food_Online_Del') && get_option('fdoe_enable_delivery_switcher', 'no') !== 'no' && get_option('fdoe_enable_delivery_switcher', 'no') !== 'only_pickup' && get_option('fdoe_disable_checkout_validation','no') !== 'yes' && (get_option('fdoe_skip_address_validation', 'no') !== 'yes' &&  'skip' !== WC()->session->get('fdoe_bypass_validation',false))) {
      return;
    }
      if( class_exists('Food_Online_Del') &&  get_option('fdoe_enable_delivery_switcher', 'no') !== 'no' &&  get_option('fdoe_enable_delivery_switcher', 'no') !== 'only_delivery' && ('local_pickup' == WC()->session->get('fdoe_shipping') || 'eathere' == WC()->session->get('fdoe_shipping')) ){
      return;
    }
    
     


    $country_pos = null;
  if(get_option('szbd_precise_address','no') != 'no' ){

    $request = wp_remote_get( SZBD_PLUGINDIRURL . 'assets/json/countries.json' );


    if( !is_wp_error( $request ) ) {
        $body = wp_remote_retrieve_body( $request );
    $country_pos = json_decode( $body );
    }
  }
  $to_localize = array(
        'countries' =>   $country_pos,
        'checkout_string_1' => __('There are no shipping options available. Please ensure that your address has been entered correctly, or contact us if you need any help.', 'woocommerce') ,
        'checkout_string_2' => __('Minimum order value is', 'szbd') ,
        'checkout_string_3' => __('You are too far away. We only make deliveries within', 'szbd') ,
        'checkout_string_4' => __('Some items in your cart don’t ship to your location', 'szbd') ,
        'cart_string_1' => __('More shipping alternatives may exist when a full shipping address is entered.', 'szbd') ,

        'no_marker_error' => __('You have to precise a location at the map', 'szbd') ,
        'store_address' => get_option('szbd_store_address_mode', 'geo_woo_store') == 'pick_store_address' ? json_decode(get_option('SZbD_settings_test', '') , true) : SZBD::get_store_address() ,

        'debug' => get_option('szbd_debug', 'no') == 'yes' ? 1 : 0,
        'deactivate_postcode' => get_option('szbd_deactivate_postcode', 'no') == 'yes' ? 1 : 0,
        'select_top_method' => get_option('szbd_select_top_method', 'no') == 'yes' ? 1 : 0,
        'store_address_picked' => get_option('szbd_store_address_mode', 'geo_woo_store') == 'pick_store_address' ? 1 : 0,
        'precise_address' => get_option('szbd_precise_address','no'),
        'nonce' => wp_create_nonce( 'szbd-script-nonce' ),
        
        'is_checkout' => is_checkout() ? 1 : 0 ,
        'is_custom_types' => get_option('szbd_types_custom','no') == 'yes' ? 1 : 0 ,
        'result_types' => get_option('szbd_result_types', array(
                        "establishment",
                        "subpremise",
                        "premise",
                        "street_address",
                        "plus_code"
                    )) ,
        'no_map_types' => get_option('szbd_no_map_types', array(
                        "establishment",
                        "subpremise",
                        "premise",
                        "street_address",
                        "plus_code",
                        "route",
                        "intersection"
                    )) ,


      );

      if (( is_checkout()  ) && get_option('szbd_deactivate_google', 'no') == 'no') {


      $google_api_key = get_option('szbd_google_api_key', '');

      wp_enqueue_script('szbd-google-autocomplete-2', 'https://maps.googleapis.com/maps/api/js?v=3&libraries=geometry,places&types=address' . '' . '&key=' . $google_api_key);
       if (WP_DEBUG === true) {
      wp_enqueue_script('shipping-del-aro', SZBD_PLUGINDIRURL . 'assets/szbd-checkout-map.js', array(
        'jquery',
        'wc-checkout',
        'szbd-google-autocomplete-2',
        'underscore'
      ) , SZBD_VERSION, true);
       }else{
         wp_enqueue_script('shipping-del-aro', SZBD_PLUGINDIRURL . 'assets/szbd-checkout-map.min.js', array(
        'jquery',
        'wc-checkout',
        'szbd-google-autocomplete-2',
        'underscore'
      ) , SZBD_VERSION, true);
       }


      wp_localize_script('shipping-del-aro', 'szbd', $to_localize);
    wp_enqueue_style('shipping-del-aro-style', SZBD_PLUGINDIRURL . '/assets/szbd-checkout-map.css', array() , SZBD_VERSION);

    }
    else if ((is_checkout() ) && get_option('szbd_deactivate_google', 'no') == 'yes') {

   if (WP_DEBUG === true) {
      wp_enqueue_script('shipping-del-aro', SZBD_PLUGINDIRURL . '/assets/szbd-checkout-map.js', array(
        'jquery',
        'wc-checkout',
        'underscore'

      ) , SZBD_PREM_VERSION, true);
   }else{
      wp_enqueue_script('shipping-del-aro', SZBD_PLUGINDIRURL . '/assets/szbd-checkout-map.min.js', array(
        'jquery',
        'wc-checkout',
        'underscore'

      ) , SZBD_PREM_VERSION, true);
   }
      wp_localize_script('shipping-del-aro', 'szbd', $to_localize);
     
 wp_enqueue_style('shipping-del-aro-style', SZBD_PLUGINDIRURL . '/assets/szbd-checkout-map.css', array() , SZBD_PREM_VERSION);
    }
            
        }

    public static function enqueue_shortcode_scripts()
      {



        $deps = array(
          'jquery',
          'underscore'
        );
        if (get_option('szbd_deactivate_google', 'no') == 'no')
          {

          $google_api_key = get_option('szbd_google_api_key', '');
          wp_register_script('szbd-script', '//maps.googleapis.com/maps/api/js?key=' . $google_api_key . '&libraries=geometry,places,drawing', array(
            'jquery'
          ) , SZBD_VERSION, true);


          if (wp_script_is('szbd-google-autocomplete-2', 'enqueued'))
            {

            $deps[] = 'szbd-google-autocomplete-2';
            }
          else if (wp_script_is('fdoe-google-autocomplete', 'enqueued'))
            {
            $deps[] = 'fdoe-google-autocomplete';

            } else if (wp_script_is('cma-google-autocomplete', 'enqueued'))
            {
            $deps[] = 'cma-google-autocomplete';

            }
            else if (wp_script_is('cmp-google-script', 'enqueued'))
            {
            $deps[] = 'cmp-google-script';

            }

          else
            {
            wp_enqueue_script('szbd-script');
            $deps[] = 'szbd-script';
            }

          }
           if( WP_DEBUG === true ) {

            wp_register_script('szbd-script-short', SZBD_PLUGINDIRURL . '/assets/szbd-shortcode.js', $deps, SZBD_VERSION, true);
            wp_enqueue_style('szbd-style-shortcode', SZBD_PLUGINDIRURL . '/assets/style-shortcode.css', array() , SZBD_VERSION);

           }else{
            wp_register_script('szbd-script-short', SZBD_PLUGINDIRURL . '/assets/szbd-shortcode.min.js', $deps, SZBD_VERSION, true);
            wp_enqueue_style('szbd-style-shortcode', SZBD_PLUGINDIRURL . '/assets/style-shortcode.min.css', array() , SZBD_VERSION);

           }
        wp_enqueue_script('szbd-script-short');
        wp_localize_script( 'szbd-script-short', 'szbd_map_monitor', array('monitor' => get_option('szbd_monitor','no') == 'yes' ? 1 :0  ) );


      }
    public function init()
      {
      // check if environment is ok
      if (self::get_environment_warning())
        {
        return;
        }
      $this->includes();
      }

    // Includes plugin files
    public function includes()
      {
      if (is_admin())
        {
        require_once (SZBD_PLUGINDIRPATH . 'classes/class-szbd-settings.php');

        require_once (SZBD_PLUGINDIRPATH . 'classes/class-szbd-admin.php');
        $this->admin = new SZBD_Admin();
        }
      require_once (SZBD_PLUGINDIRPATH . 'includes/szbd-template-functions.php');
      require_once (SZBD_PLUGINDIRPATH . 'classes/class-szbd-google-server-request.php');
      require_once (SZBD_PLUGINDIRPATH . 'classes/class-szbd-ajax.php');
      require_once (SZBD_PLUGINDIRPATH . 'classes/class-szbd-helping-classes.php');  
      require_once (SZBD_PLUGINDIRPATH . 'classes/class-szbd-shippingmethod.php');
      require_once (SZBD_PLUGINDIRPATH . 'classes/class-szbd-the-post.php');
       new Sbdzones_Post();
      }
    public function init_shortcode()
      {

      if (!is_admin() && !is_ajax() && !self::get_environment_warning())
        {
        require_once (SZBD_PLUGINDIRPATH . 'classes/class-szbd-shortcode.php');
        $this->shortcode = SZBD_Shortcode::instance();

        }
      }
    // For use in future versions. Loads text domain files
    public function load_text_domain()
      {
      load_plugin_textdomain('szbd', false, dirname(plugin_basename(__FILE__)) . '/languages/');
      }

    public function insert_to_checkout()
      {
       if (!is_checkout() || WC()
        ->cart
        ->needs_shipping() === false)
        {
        return;
        }

      if (class_exists('Food_Online_Del') && get_option('fdoe_enable_delivery_switcher', 'no') !== 'no' && get_option('fdoe_enable_delivery_switcher', 'no') !== 'only_pickup' && get_option('fdoe_disable_checkout_validation','no') !== 'yes' && (get_option('fdoe_skip_address_validation', 'no') !== 'yes' &&  'skip' !== WC()->session->get('fdoe_bypass_validation',false))) {
      return;
    }
      if( class_exists('Food_Online_Del') &&  get_option('fdoe_enable_delivery_switcher', 'no') !== 'no' &&  get_option('fdoe_enable_delivery_switcher', 'no') !== 'only_delivery' && ('local_pickup' == WC()->session->get('fdoe_shipping') || 'eathere' == WC()->session->get('fdoe_shipping')) ){
      return;
    }

      $class = '';
      $option = get_option('szbd_precise_address', 'no');
      if ($option == 'no')
        {
        return;
        }
      else if ($option == 'at_fail')
        {
        $class = 'szbd-hide';
        }
        if(get_option('szbd_precise_address_mandatory', 'no') == 'yes'){

            $string = __('Please Precise Your Location', 'szbd');

        }else{

            $string = __('Precise Address?', 'szbd');

        }
    ob_start();
      echo '<div id="szbd_checkout_field" class="shop_table ' . $class . '"><h3>' . $string . '</h3>';
 if(get_option('szbd_precise_address_plus_code', 'no') == 'yes'){
             self::insert_plus_code_to_checkout();
            }
      woocommerce_form_field('szbd-picked', array(
        'type' => 'text',
        'class' => array(
          'szbd-hidden'
        ) ,
        'label' => __('Precise Address?', 'szbd') ,

      ));
      woocommerce_form_field('szbd-map-open', array(
        'type' => 'checkbox',
        'class' => array(
          'szbd-hidden'
        ) ,


      ));

      echo '<div id="szbd-pick-content"><div class="szbd-checkout-map"><div id="szbd_map"></div></div></div></div>';
      echo ob_get_clean();  

      }
        function szbd_disable_shipping_calc_on_cart($show_shipping) {

    if (is_cart() && get_option('szbd_hide_shipping_cart', 'no') == 'yes') {
      return false;
    }
    return $show_shipping;
  }
       public static function insert_plus_code_to_checkout(){
             
         woocommerce_form_field('szbd-plus-code', array(
                    'type' => 'text',
                   
                    'class' => array(
                    'szbd-plus-code-form'
                ) ,
                    'label' => __('Find Location with Google Plus Code', 'szbd') ,
                    'placeholder' => __('Enter Plus Code...', 'szbd') ,
                ));
        }
      function validate_checkout_field_process() {

    if (( get_option('szbd_precise_address_mandatory', 'no') == 'yes' &&  get_option('szbd_precise_address', 'no') == 'always' && isset($_POST['szbd-picked']) && ! $_POST['szbd-picked'] ) ||
        ( get_option('szbd_precise_address_mandatory', 'no') == 'yes' &&  get_option('szbd_precise_address', 'no') == 'at_fail' && isset($_POST['szbd-picked']) && ! $_POST['szbd-picked'] && isset($_POST['szbd-map-open']) && 1 == $_POST['szbd-map-open'] )
        ){

        wc_add_notice( __( 'Please precise your address at the map','szbd' ), 'error' );
    }
    }

    // Add setting links to Plugins page
    public function add_action_links($links)
      {
      if (plugin_basename(__FILE__) == "shipping-zones-by-drawing-for-woocommerce/shipping-zones-by-drawing.php")
        {
        $links_add = array(
          '<a href="' . admin_url('admin.php?page=wc-settings&tab=szbdtab') . '">Settings</a>',
          '<a target="_blank" href="https://shippingzonesplugin.com/">Go Premium</a>'
        );
        }
      else
        {
        $links_add = array(
          '<a href="' . admin_url('admin.php?page=wc-settings&tab=szbdtab') . '">Settings</a>'
        );
        }
      return array_merge($links, $links_add);
      }

    // Checks if WooCommerce etc. is active and if not returns error message
    static function get_environment_warning()
      {
      include_once (ABSPATH . 'wp-admin/includes/plugin.php');
      if (!defined('WC_VERSION'))
        {
        return __('Shipping Zones by Drawing requires WooCommerce to be activated to work.', 'szbd');
        die();
        }
      //if this is Premium
      /*
      else if (is_plugin_active('shipping-zones-by-drawing-for-woocommerce/shipping-zones-by-drawing.php'))
      {
      return __('Shipping Zones by Drawing Premium can not be activated when the free version is active.', 'szbd');
      die();
      }*/
      // If this is free version
      else if (is_plugin_active('shipping-zones-by-drawing-premium/shipping-zones-by-drawing.php'))
        {
        return __('Shipping Zones by Drawing can not be activated when the premuim version is active.', 'szbd');
        die();
        }
      return false;
      }

    // Checks if environment is ok
    public function check_environment()
      {
      $environment_warning = self::get_environment_warning();
      if ($environment_warning && is_plugin_active(plugin_basename(__FILE__)))
        {
        $this->add_admin_notice('bad_environment', 'error', $environment_warning);
        deactivate_plugins(plugin_basename(__FILE__));
        }
      }

    // Adds notice if environmet is not ok
    public function add_admin_notice($slug, $class, $message)
      {
      $this->notices[$slug] = array(
        'class' => $class,
        'message' => $message
      );
      }

    public function admin_notices()
      {
      foreach ((array)$this->notices as $notice_key => $notice)
        {
        echo "<div class='" . esc_attr($notice['class']) . "'><p>";
        echo wp_kses($notice['message'], array(
          'a' => array(
            'href' => array()
          )
        ));
        echo '</p></div>';
        }
      unset($notice_key);
      }

    function posts_columns_id($defaults)
      {
      $defaults['szbd_post_id'] = __('ID');
      return $defaults;
      }
    function posts_custom_id_columns($column_name, $id)
      {
      if ($column_name === 'szbd_post_id')
        {
        echo $id;
        }
      }

    
    function szbd_checkout_field_update_order_meta($order_id)
      {
      if (isset($_POST['szbd-picked']) && !empty($_POST['szbd-picked']) && $_POST['szbd-picked'] != '')
        {

        update_post_meta($order_id, 'szbd_picked_delivery_location', sanitize_text_field($_POST['szbd-picked']));
        }
         if (isset($_POST['szbd-plus-code']) && !empty($_POST['szbd-plus-code']) && $_POST['szbd-plus-code'] != '') {
                $meta = sanitize_text_field($_POST['szbd-plus-code']);
                update_post_meta($order_id, 'szbd_picked_delivery_location_plus_code', $meta);
            }
      }
      static function szbd_get_plus_code($order_id){
            $output = '';
            $meta = get_post_meta($order_id , 'szbd_picked_delivery_location_plus_code', true);
            if (is_string($meta) && $meta != '' && !empty($meta) && $meta !== false) {
                
                $output .= '<p><strong>' . __('Plus Code', 'szbd') . ': </strong>'. $meta .'</p>';
            }
            return $output;
        }
  function szbd_show_checkout_field_admin_order_meta($order)
      {
      $meta = get_post_meta($order->get_id() , 'szbd_picked_delivery_location', true);
      if (is_string($meta) && $meta != '' && !empty($meta) && $meta !== false)
        {
            $location = json_decode($meta);
        if ($location !== null && isset($location->lat))
          {
             $lat = $location->lat;
            $long = $location->lng;
             echo '<p><strong>' . __('Picked Delivery Location', 'szbd') . ':</strong> ';
         echo '<a target="_blank" href="https://www.google.com/maps/search/?api=1&query=' . $lat . ',' . $long . '" ><br>' . __('Open delivery location with Google Maps', 'szbd') . '</a></p>';
         echo self::szbd_get_plus_code($order->get_id());
        }
        }
      }
    function szbd_add_picked_location_to_emails($order, $sent_to_admin, $plain_text, $email)
      {
      $meta = get_post_meta($order->get_id() , 'szbd_picked_delivery_location', true);
      if (is_string($meta) && $meta != '' && !empty($meta) && $meta !== false)
        {
        $location = json_decode($meta);
        if ($location !== null && isset($location->lat))
          {

          if ($email->id == 'customer_processing_order')
            {

            $lat = $location->lat;
            $long = $location->lng;
            echo '<p><strong>' . __('Picked Delivery Location', 'szbd') . ':</strong> ';
            
            echo '<a target="_blank" href="https://www.google.com/maps/search/?api=1&query=' . $lat . ',' . $long . '" ><br>' . __('Open delivery location with Google Maps', 'szbd') . '</a></p>';
            echo self::szbd_get_plus_code($order->get_id());
            }
          if ($email->id == 'customer_completed_order')
            {

            $lat = $location->lat;
            $long = $location->lng;
            echo '<p><strong>' . __('Picked Delivery Location', 'szbd') . ':</strong>';
            echo '<a target="_blank" href="https://www.google.com/maps/search/?api=1&query=' . $lat . ',' . $long . '" ><br>' . __('Open delivery location with Google Maps', 'szbd') . '</a></p>';
            echo self::szbd_get_plus_code($order->get_id());
            }
          if ($email->id == 'new_order')
            {

            $lat = $location->lat;
            $long = $location->lng;
            echo '<p><strong>' . __('Picked Delivery Location', 'szbd') . ':</strong>';
            echo '<a target="_blank" href="https://www.google.com/maps/search/?api=1&query=' . $lat . ',' . $long . '" ><br>' . __('Open delivery location with Google Maps', 'szbd') . '</a></p>';
            echo self::szbd_get_plus_code($order->get_id());
            }
          }
        }
      }
    function szbd_add_picked_location_to_thankyou_page($order_id)
      {
      $meta = get_post_meta($order_id, 'szbd_picked_delivery_location', true);
      if (is_string($meta) && $meta != '' && !empty($meta) && $meta !== false)
        {

        $location = json_decode($meta);
        if ($location !== null && isset($location->lat))
          {
          $lat = $location->lat;
          $long = $location->lng;
          echo '<p><strong>' . __('Picked Delivery Location', 'szbd') . ':</strong> ';
          echo '<a target="_blank" href="https://www.google.com/maps/search/?api=1&query=' . $lat . ',' . $long . '" ><br>' . __('Open delivery location with Google Maps', 'szbd') . '</a></p>';
           echo self::szbd_get_plus_code($order_id);
          }
        }
      }

         function szbd_preview_meta(){
		?><#

		_.each(data.data.meta_data,function(el,i){



						if ( el.key == 'szbd_picked_delivery_location' ) {
						try{



									let obj = jQuery.parseJSON(el.value);


									var url = 'https://www.google.com/maps/search/?api=1&query=' + obj.lat + ',' + obj.lng;
							}catch(err){}
									if(typeof url !== 'undefined'){
									#>
								<div class="wc-order-preview-addresses">
							<div class="wc-order-preview-address">
									<h2><?php esc_html_e( 'Picked delivery location', 'szbd' ); ?></h2>

									<a target="_blank" href="{{ url }}" ><?php esc_html_e('Open delivery location with Google Maps','szbd') ?> </a>

								</div></div>
							<# }}






							});


							#>


							<?php

	}
   public static function get_cart_products() {
            $products = array();
            foreach (WC()
                ->cart
                ->get_cart_contents() as $cart_item) {
                $product = wc_get_product($cart_item['product_id']);
                if ($product !== null) {
                    $products[] = $product;
                }
            }
            return empty($products) ? null : $products;
        }
        public static function is_cart_ok($cats_) {
            $products = isset(self::$products) ? self::$products : self::get_cart_products();
            $cats = $cats_;
            if (is_array($products) && is_array($cats)) {
                foreach ($products as $product) {
                    if (empty(array_intersect($product->get_category_ids() , $cats))) {
                        return false;
                    }
                }
            }
            return true;
        }
        // Used if to filter out shipping methods with non allowed categories
        public static function szbd_filter_shipping_methods_for_checkout($rates) {
            $ok_methods = array();
            foreach ($rates as $rate_id => $rate) {
                if ($rate->method_id == 'szbd-shipping-method') {
                    $shipping_class_names = WC()
                        ->shipping
                        ->get_shipping_method_class_names();
                    $method_instance = new $shipping_class_names['szbd-shipping-method']($rate->get_instance_id());
                   
                 if ((empty($method_instance->ok_categories) || self::is_cart_ok($method_instance->ok_categories) )) {

                        $ok_methods[$rate_id] = $rate;
                    }
                    else {
                        continue;
                    }
                }
                else {
                    $ok_methods[$rate_id] = $rate;
                }
            }
            return $ok_methods;
        }
        // Used if to filter out shipping methods at server mode
        public function szbd_filter_shipping_methods_for_checkout_server_mode($rates) {
      
          
           if(  is_cart() && get_option('szbd_enable_at_cart','no') == 'no'){
                
               return $rates;
            }
            
            $ok_methods = array();
            foreach ($rates as $rate_id => $rate) {
                if ($rate->method_id == 'szbd-shipping-method') {
                    $shipping_class_names = WC()
                        ->shipping
                        ->get_shipping_method_class_names();
                    $method_instance = new $shipping_class_names['szbd-shipping-method']($rate->get_instance_id());
                  
                 
                  
                  
                   
                    if ( 
                        
                        (empty($method_instance->ok_categories) || self::is_cart_ok($method_instance->ok_categories) ) &&
                        szbd_polygonContainsPoint($method_instance)  &&
                       szbd_radiusIsOk($method_instance) 
                      
                        
                        
                        ) {
                      
                         $ok_methods[$rate_id] = $rate;
                     
                     
                     
                    }
                    
                    else {
                        continue;
                    }
                }
                else {
                    $ok_methods[$rate_id] = $rate;
                }
            }
            
            return $ok_methods;
        }
         public static function get_customer_address_string($package, $separator = ' ') {
        $package['destination']['postcode'] = wc_format_postcode($package['destination']['postcode'] , $package['destination']['country'] );

        $formatted_address_string =  WC()->countries->get_formatted_address( $package['destination'] , $separator );
          return $formatted_address_string;
        }

static function get_store_address($format = 'array') {
   
          if (!isset(self::$store_address)) {
            $store_address = get_option('woocommerce_store_address', '');
            $store_address_2 = get_option('woocommerce_store_address_2', '');
            $store_city = get_option('woocommerce_store_city', '');
           

            $store_raw_country = get_option('woocommerce_default_country', '');
            $split_country = explode(":", $store_raw_country);
            // Country and state
            $store_country = $split_country[0];
             $store_postcode =  wc_format_postcode(get_option('woocommerce_store_postcode', ''), $store_country);
            // Convert country code to full name if available
            if (isset(WC()
              ->countries
              ->countries[$store_country])) {
              $store_country = WC()
                ->countries
                ->countries[$store_country];
            }
            $store_state = isset($split_country[1]) ? $split_country[1] : '';
            $store_loc = array(
              'store_address' => $store_address,
              'store_address_2' => $store_address_2,
              'store_postcode' => $store_postcode,
              'store_city' => $store_city,

              'store_state' => $store_state,
              'store_country' => $store_country,
               'store_country_code' => $split_country[0],

            );
            self::$store_address = $store_loc;
          }
           if($format == 'comma_seperated'){
            
            $address = array(
			
			'address_1'  => self::$store_address['store_address'],
			'address_2'  => self::$store_address['store_address_2'],
			'city'       => self::$store_address['store_city'],
			'state'      => self::$store_address['store_state'],
			'postcode'   => self::$store_address['store_postcode'],
			'country'    => self::$store_address['store_country_code'],
		);
            return WC()->countries->get_formatted_address($address,',') . ', '. self::$store_address['store_country'] ;
            
           }
          return self::$store_address;
        }



    }
  }
$GLOBALS['szbd_item'] = SZBD::instance();
